*****************************************************************************************************
-- THE SQL*LOADER
*****************************************************************************************************


-- THE SQL*LOADER IS THE UTILITY TOOL USED IN ORACLE TO IMPORT TEXT (AND BINARY) DATA 
-- THERE ARE MANY OPTIONS TO SPECIFY THE DETAILS OF THE TEXT DATA (FORMAT, SEPARATORS, TRANSFORMATIONS, ETC)


-- HOWEVER, IF COMPLEX MANIPULATIONS ARE NECESSARY IT IS RECOMMEDED TO USE AN ETL (EXTRACT TRANSFORM LOAD) TOOL 

-- NOTE: SQL*LOADER CANNOT EXPORT DATA IN TEXT FORMAT
-- BUT YOU CAN USE THE SQLPLUS TO DO THAT. EXAMPLE TO EXPORT THE CONTENTS OF THE EMPLOYEES TABLE IN CSV (export.sql):

set echo off newpage 0 space 0 pagesize 0 feed off head off trimspool on
spool employees.csv
select   FIRST_NAME || ',',
  LAST_NAME || ','
FROM HR.EMPLOYEES;
spool off

-- RUN IN SQLPLUS WITH @

$sqlplus hr
>@export.sql


-- CHECK THE employees.csv

-- SOME EXAMPLES OF USING SQLLOADER TO IMPORT DATA:

***************************************
-- 1) SIMPLE LOADING OF A CSV FILE
***************************************

  -- THE DESTINATION TABLE:

create table emp
(
   empno int,
   empname varchar(100),
   sal int,
   deptno int
)

  select * from emp;

  -- THE CONTROL FILE LOADER1.CTL
  load data
  infile 'mydata.csv'
  into table emp
  fields terminated by "," optionally enclosed by '"'
  (empno, empname, sal, deptno)

  -- THE DATA FILE mydata.csv
  10001,"Scott Tiger", 1000, 40
  10002,"Frank Naude", 500, 20

  
  -- RUNNING THE SQL*LOADER
  $sqlldr hr control=loader1.ctl

  -- CHECKING THE DATA
    SELECT * FROM emp;

  -- DON'T FORGET TO CHECK THE LOADER1.LOG FILE FOR ERRORS


********************************************************************************  
-- 2) USING THE DATA INSIDE THE CONTROL FILE AND WITH POSITIONING FOR THE FIELDS
********************************************************************************

   -- THE DESTINATION TABLE:
create table dep
(
   dept char(4),
   deptname char(20)
)


  select * from dep


  -- THE CONTROL FILE LOADER2.CTL
  load data
   infile *
   replace
   into table dep
   (  dept     position (02:05) char(4),
      deptname position (08:27) char(20)
   )
  begindata
  COSC  COMPUTER SCIENCE
  ENGL  ENGLISH LITERATURE
  MATH  MATHEMATICS
  POLY  POLITICAL SCIENCE

   -- RUNNING THE SQL*LOADER
  $sqlldr hr control=loader2.ctl

    -- CHECKING THE DATA
    SELECT * FROM dep;



  -- DON'T FORGET TO CHECK THE LOADER2.LOG FILE FOR ERRORS

********************************************************************************  
-- 3) MODIFYING THE DATA AS THE DATABASE GETS LOADED
********************************************************************************

  -- THE DESTINATION TABLE:
CREATE TABLE modified_data
(
   region INT,
   time_loaded varchar(100),
   data1       INT,
   data2 VARCHAR(100),
   data3 DATE
)

  SELECT * FROM modified_data
  -- THE CONTROL FILE LOADER3.CTL
    LOAD DATA
    INFILE *
    INTO TABLE modified_data
    (  
       region                      CONSTANT '31',
       time_loaded                 "to_char(SYSDATE, 'HH24:MI')",
       data1        POSITION(1:5)  ":data1/100",
       data2        POSITION(6:15) "upper(:data2)",
       data3        POSITION(16:22)"to_date(:data3, 'YYMMDD')"
    )
  BEGINDATA
  11111AAAAAAAAAA991201
  22222BBBBBBBBBB990112


  -- RUNNING THE SQL*LOADER
  $sqlldr hr control=loader3.ctl

    -- CHECKING THE DATA
    SELECT * FROM modified_data;

  -- DON'T FORGET TO CHECK THE LOADER3.LOG FILE FOR ERRORS

********************************************************************************  

-- 4) MULTIPLE FILES TO A SINGLE TABLE

********************************************************************************

  -- THE DESTINATION TABLE:
create table emp_multi
(
   empno int,
   empname varchar(100),
   sal int,
   deptno int
)

  select * from emp_multi;

  -- THE CONTROL FILE LOADER4.CTL
  load data
  infile 'mydata1.csv'
  infile 'mydata2.csv'
  infile 'mydata3.csv'
  into table emp_multi
  fields terminated by "," optionally enclosed by '"'
  (empno, empname, sal, deptno)

  -- THE DATA FILE mydata1.csv
  10001,"Scott Tiger", 1000, 40
  10002,"Frank Naude", 500, 20

    -- THE DATA FILE mydata2.csv
  20001,"Jonh Smith", 1000, 40
  20002,"Ellen Cloud", 500, 20

    -- THE DATA FILE mydata3.csv
  30001,"Rick Jones", 1000, 40
  30002,"Eva Lopez", 500, 20


  -- RUNNING THE SQL*LOADER
  $sqlldr hr control=loader4.ctl

  -- CHECKING THE DATA
    SELECT * FROM emp_multi;


********************************************************************************  
-- 5) LOADING DATA TO MULTIPLE TABLES
********************************************************************************

   -- THE DESTINATION TABLES:
CREATE TABLE tab1
(
   tab char(2),
   col1 INTEGER
)

CREATE TABLE tab2
(
   tab char(2),
   col1 INTEGER
)

   -- THE CONTROL FILE LOADER5.CTL
    LOAD DATA
     INFILE *
     INTO TABLE tab1 WHEN tab = 'tab1'
       ( 
         tab  FILLER CHAR(4),
         col1 INTEGER
       )
     INTO TABLE tab2 WHEN tab = 'tab2'
       ( 
         tab  FILLER POSITION(1:4),
         col1 INTEGER
       )
    BEGINDATA
    tab1|1
    tab1|2
    tab2|2
    tab3|3

  -- RUNNING THE SQL*LOADER
  $sqlldr hr control=loader5.ctl

    -- CHECKING THE DATA
    SELECT * FROM tab1
    SELECT * FROM tab2


********************************************************************************  
-- 6) LOADING IMAGE AND OTHER BINARY FILES
********************************************************************************

   -- THE DESTINATION TABLE:
CREATE TABLE image_table
(
   image_id   INTEGER,
   file_name  CHAR(100),
   image_data BLOB
)
      
    -- THE CONTROL FILE LOADER6.CTL
    LOAD DATA
    INFILE *
    INTO TABLE image_table
    APPEND
    FIELDS TERMINATED BY ','
    (
           image_id   INTEGER,
           file_name  CHAR(100),
          image_data LOBFILE (file_name) TERMINATED BY EOF
    )
    BEGINDATA
    001,/home/mauro/oracle25/image1.gif
    002,/home/mauro/oracle25/image2.jpg
    003,/home/mauro/oracle25/image3.jpg

    -- RUNNING THE SQL*LOADER
    $sqlldr hr control=loader6.ctl

    -- CHECK THE BINARY DATA
    select utl_raw.cast_to_varchar2(DBMS_LOB.SUBSTR(image_data, 2000, 1)) from image_table

